/* This class is used to scan all the files of the file system or under some
 * particular directory that user defined. 
 */


import java.io.*;

public class FileScan {

     // the starting root for file categorization
    private File _root;
    
    //=====================================================================

    /* default constructor, it will use root as the beginning scan point */
    public FileScan() 
    {
	// create File instance of root "/"
	_root = new File("/");

	// scan all the files amd add file info to file info vector
	_scan_files(_root);
    }

    /* constructor which takes parameter and begins from the provided dir */

    public FileScan(String subtree_root)
    {
	/* create File instance of the subtree root and if it does not 
	 * exist, return error */
	_root = new File(subtree_root);

	if (!_root.exists())
	    System.out.println("Error: Illegal starting point for file"+ 
                               " categorization.");
	
	// scan all the files amd add file info to file info vector
	_scan_files(_root);
    }

    /* get the starting tree root for file categorization */
    public String getRoot()
    {
	return _root.getAbsolutePath();
    }


    /* this method will scan all the files under _root */
    private void _scan_files(File subroot)
    {

	/* the root is a file */
	if (subroot.isFile()) {
	    // get the path 
	    String path = subroot.getAbsolutePath();
	  
	    // add file info entry to the file info vector
	    ourDB.fileInfoV.add(new FileInfoEntry(subroot.getName(), path,
				checkType(subroot))); 
	}
	else 
	{
	    String[] file_list = subroot.list();
		
	    for (int i=0; i<file_list.length; i++) 
		_scan_files(new File(subroot,file_list[i]));
	}
    }
		
    /* This method will check the type of the files */
    public static String checkType(File file)
    {
	String ext = _get_ext(file);

	// check whether the ext reflect the true type of the file
	if (0 == ext.length()) {
	    // check whether it is a email file
	    if (true == _check_email(file))
		return "email";
	    else 
		return "";
	}
	// 1. PS file
	else if (ext.equalsIgnoreCase("ps")) {
	    if (true == _check_ps(file))
		return "ps";
	    else 
		return "ps_nomatch";
	}
	// 2. PDF file
	else if (ext.equalsIgnoreCase("pdf")) {
	    //if (true == _check_pdf(file))
	    return "pdf";
	    //else 
	    //return "pdf_nomatch";
	}
	// 3. Word file
	else if (ext.equalsIgnoreCase("doc")) {
	    return "doc";
	}
	// 4. JPEG file
        else if (ext.equalsIgnoreCase("jpg")) {
	    return "jpg";
	}
	// 5. BMP file
	else if (ext.equalsIgnoreCase("bmp")) {
	    return "bmp";
	}
	// 6. TIF file
	else if (ext.equalsIgnoreCase("tif")) {
	    return "tif";
	}
	// 7. GIF file
	else if (ext.equalsIgnoreCase("gif")) {
	    return "gif";
	}
	// 8. PPS file
	else if (ext.equalsIgnoreCase("pps")) {
	    return "pps";
	}
	// 9. EXE file
	else if (ext.equalsIgnoreCase("exe")) {
	    return "exe";
	}
	// 10. CLASS file
	else if (ext.equalsIgnoreCase("class")) {
	    return "class";
	}
	// 11. O file
	else if (ext.equalsIgnoreCase("o")) {
	    return "o";
	}
	// 12. OUT file
	else if (ext.equalsIgnoreCase("out")) {
	    return "out";
	}
	// 13. JAVA file
	else if (ext.equalsIgnoreCase("java")) {
	    return "java";
	}
	// 14. C file
	else if (ext.equals("c")) {
	    return "c";
	}
	// 15. C++ file
	else if (ext.equals("C")) {
	    return "c++";
	}
	// 16. Another extension of C++ file
	else if (ext.equalsIgnoreCase("cpp")) {
	    return "c++";
	}
	// 17. TXT file
	else if (ext.equalsIgnoreCase("txt")) {
	    return "txt";
	}
	// 18. H file
	else if (ext.equalsIgnoreCase("h")) {
	    return "h";
	}
	// 19. unrecgnized extension
	else {
	    return ext;
	}
    }
	   
    /* This method is used to extract the extension part from the file name .
     * If the file has no extension, return NOEXT;
     */
    private static String _get_ext(File file)
    {
	// get the file name first
	String file_name = file.getName();
	
	// check the extension
	int dot_pos = file_name.indexOf((int)'.');
	
	if (0 == dot_pos) {
	    file_name = file_name.substring(1);
	    dot_pos = file_name.indexOf('.');
	}

	if (-1 == dot_pos) {
	    return "";
	}
	else {
	    // find the senond dot position
	    int dot2_pos = file_name.indexOf((int)'.',dot_pos+1);
	    if (-1 != dot2_pos) {
		if ((file_name.substring(dot2_pos+1)).equalsIgnoreCase("bak") ||
		    (file_name.substring(dot2_pos+1)).equalsIgnoreCase("swp")) {
		    file_name = file_name.substring(0, dot2_pos);
		}
		else {
		    dot_pos = dot2_pos;
		}
	    }

	    // extract the extension part
	    String ext = file_name.substring(dot_pos+1);
	    // get rid of the '~' character for emacs
	    if (ext.endsWith("~"))
		ext = ext.substring(0,ext.length()-1);
	    
	    return ext;
	}
		
    }

    // confirm whether the file is a PS file
    private static boolean _check_ps(File file) {
	
	String line = null; // string used to read the file content

	try {

	    BufferedReader file_in = new BufferedReader(new FileReader(file.getAbsolutePath()));
	
	    // Check first line, whether it begins with "%!PS"
	    line = file_in.readLine();
	}
	catch (Exception e) { 
	    System.out.println("ERROR: " + e.toString());
	    System.exit(0);
	}

	if (!line.startsWith("%!PS")) 
	    return false;
	else 
	    return true;

	    // check the second line, whether it is started with "%%Creator"
	    //line = file_in.readLine();
	    //if (!line.startsWith("%%Creator")) 
	    //return false;
	    
	    // check the third line, whether it is started with "%%Title"
	    //line = file_in.readLine();
	    //if (!line.startsWith("%%Title")) 
	    //return false;
	    
	    // check the line at the end
	    //String line1, line2, line3;
	    //line1 = file_in.readLine();
	    //line2 = file_in.readLine();
	    //while (true) {
	    //line3 = file_in.readLine();
		
	    //if (line1.startsWith("userdict /end-hook") && line2.equals("%%EOF") &&
	    //    null == line3)
	    //    return true;
	    //else if (null == line3)
	    //    return false;
		
	    //line1 = line2;
	    //line2 = line3;
	    //}
    }

    // check whether the file is a pdf file
    private static boolean _check_pdf(File file) {

	String line = null; // string used to read the file content

	try {
	    BufferedReader file_in = new BufferedReader(new FileReader(file.getAbsolutePath()));
	
	    // Check first line, whether it begins with "%PDF"
	    line = file_in.readLine();
	}
	catch (Exception e) { 
	    System.out.println("ERROR: " + e.toString());
	    System.exit(0);
	}

	if (!line.startsWith("%!PDF")) 
	    return false;
	else 
	    return true;
    }

    // check whether a file without extension is an email file
    private static boolean _check_email(File file) {

	String line1,line2,line3,line4; // strings used to read the file content

	try {
	    BufferedReader file_in = new BufferedReader(new FileReader(file.getAbsolutePath()));
	    
	    line1 = file_in.readLine();
	    if (null == line1)
		return false;
	    line2 = file_in.readLine();
	    if (null == line2)
		return false;
	    line3 = file_in.readLine();
	    if (null == line3)
		return false;
	    line4 = file_in.readLine();
	    if (null == line4)
		return false;

	    while (true) {
		if (line1.startsWith("Date:") && line2.startsWith("From:") &&
		    line3.startsWith("To:")   && line4.startsWith("Subject:"))
		    return true;
		else {
		    line1 = line2;
		    line2 = line3;
		    line3 = line4;

		    line4 = file_in.readLine();
		    if (null == line4)
			return false;
		}
	    }

	}
	catch (Exception e) { 
	    System.out.println("ERROR: " + e.toString());
	    System.exit(0);
	}

	return false;
    }

    /*  public static void main(String[] args) 
    {
	FileScan fs = new FileScan("/afs/cs.wisc.edu/u/w/j/wjin/course/cs736/project");

	ourDB.insertFileInfo("afs/cs.wisc.edu/u/w/j/wjin/course/cs736/project/a.C");
	ourDB.delFileInfo(11);
	ourDB.insertFileInfo("afs/cs.wisc.edu/u/w/j/wjin/course/cs736/project/a.C");
	ourDB.delFileInfo(20);
	ourDB.delFileInfo(13);
	ourDB.insertFileInfo("afs/cs.wisc.edu/u/w/j/wjin/course/cs736/project/a.C");
	ourDB.dumpFileInfoV();
	ourDB.dumpFileNumFreeList();
    }

    */


}
