import java.util.Vector;
import java.io.Serializable;

class Leaf implements Serializable {

    static final int MAX = 4;
    static final int MIN = MAX/4;

    // the _keys will be vector of vectors
    String[] _keys ;
    // the _values will also be vector of vectors, and the
    // position of the _values corresponds to the position
    // in the vector of their key.
    Vector[] _values;

    Leaf _next;
    
    
    int _arity ; //number of keys in this node currently
    
    public Leaf(){

        _keys = new String[MAX];
        _values = new Vector[MAX]; 
        _arity = 0;
        _next = null;
    }
    
    public Leaf(String[] keys,Vector[] values, int arity) {
        _keys = keys;
        _values = values;
        _arity = arity;
        _next = null;
    }

    public Leaf(String[] keys, Vector[] values, int arity, Leaf next) {
         _keys = keys;
        _values = values;
        _arity = arity;
        _next = next;
    }
        

    public Object lookup(String key) {

        for(int i = 0; i < _arity; i++) {
  System.out.println(" leaf key = " + _keys[i]);
            if (_keys[i].compareTo(key) ==0)
                return _values[i];
        }

        return null;
    }


    // split the leaf when it exceeds the Max size
    public NodeEntry split(LeafEntry entry, int position) {
System.out.println("~~~~~~~~~~~~~IN SPLIT~~~~~~~~~~~~~");
      
        //create a new leaf node and let its next point to the leaf node
        //that was pointed to by the current node.
      
        Leaf newLeaf = new Leaf();
        
        newLeaf._next = _next;
        _next = newLeaf;

        int oldArity = _arity;
        int newArity = 0;
        int newSize = MAX/2;
        int moveIndex = (position < MAX/2)? MAX/2-1 : MAX/2;

        //find the node entry to be promoted up
        // if the position of the new node to be added is just
        // in the middle, then its key and the associated newly
        // created split node is the new node entry,
        // if the position of th new node to be added is within
        // the scope of old entry, then it means that all the
        // rest of the entries will be shifted one place backward
        // thus making the node entry at [MAX/2-1] to be the key that
        // is going to be promoted.  Else the key to be promoted will
        // be the MAX/2 entry. The children pointer to be promoted will
        // be always the one that point to the new node created by split.

        // the entry to be promoted up to get inserted to the internal
        // node.
        NodeEntry middle = null;
        // if the position of the newly added node is the middle one
        // then promote this value and the new created node as the entry
        // to be promoted

        int index = 0;
        if (position == (MAX/2)) {
            middle = new NodeEntry(entry._key, newLeaf);

      System.out.println(" In Pos  1/2 , key promote = "+ middle._key);

            moveIndex = MAX/2;
        
            // for leaf, the value need to be both stored and pushed up 
            newLeaf._keys[0] = entry._key;
            newLeaf._values[0] = entry._values;
            newArity++;
            for (int i = moveIndex; i < MAX; i++) {
                newLeaf._keys[i-moveIndex+1] = _keys[i];
                newLeaf._values[i-moveIndex+1] = _values[i];
                _keys[i] = null;
                _values[i] = null;
                newArity++;
                oldArity--;
            }
        }
        // if the new entry to be inserted is not the middle node
        else {
            // if the new node is to be added into the old leaf
            if (position < MAX/2) {
                middle = new NodeEntry(_keys[MAX/2], newLeaf);

      System.out.println(" In Pos < 1/2 , key promote = "+ middle._key);

                moveIndex = MAX/2;
                for (int i = moveIndex; i < MAX; i++) {
                      newLeaf._keys[i-moveIndex] = _keys[i];
                      newLeaf._values[i-moveIndex] = _values[i];
                      _keys[i] = null;
                      _values[i] = null;
                      newArity++;
                      oldArity--;
                }
 
                for (int i = MAX/2; i > position ;i--) {
                    _keys[i] = _keys[i-1];
                    _values[i] = _values[i-1];
                }

                _keys[position] = entry._key;
                _values[position] = entry._values;
                oldArity++;

              } // end-if
              // if the to be added entry resides in the new leaf.
              else { 
                  middle = new NodeEntry(_keys[MAX/2+1], newLeaf);
      System.out.println(" In Pos > 1/2 , key promote = "+ middle._key);


                  moveIndex = MAX/2+1;

                  for (int i = moveIndex; i < MAX;i++) {
                      newLeaf._keys[i-moveIndex]= _keys[i];
                      newLeaf._values[i-moveIndex] = _values[i];
                      _keys[i] = null;
                      _values[i] = null;
                      oldArity--;
                      newArity++;
                  }

                  int newPos = position -moveIndex; 
                  for (int i = newArity; i > newPos;i--) {
                      newLeaf._keys[i] = newLeaf._keys[i-1];
                      newLeaf._values[i] = newLeaf._values[i-1];
                  }
                  newLeaf._keys[newPos] = entry._key;
                  newLeaf._values[newPos] = entry._values;
                  newArity++;
              } // end-else
        } //end-else

System.out.println("old arity = " + _arity);

        newLeaf._arity = newArity;;
        _arity = oldArity;

System.out.println("modified  arity = " + _arity + " new arity = " + newLeaf._arity);

        //update the next pointer for this leaf to point to the new leaf;
        _next = newLeaf;

System.out.println(" AN ENTRY GOT PROMOTED :::::");
System.out.println("          key = " + middle._key + " child  :"+middle._child);
System.out.println("End of SpLIT");
System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~");

if (!(middle._child instanceof Leaf)) {
  System.out.println("Leaf promote WRONG");
  System.exit(0);
}

        return middle;
    }    

    public void leafRemove(Vector delV) {
      
       for (int i = 0; i < _arity; i++) {
            for (int j = 0; j < delV.size(); j++) {
                for (int k = 0; k < _values[i].size(); k++) {
                    if (delV.elementAt(j) == _values[i].elementAt(k))
                        _values[i].removeElementAt(k);
                }
            }
        }

    }

           

    // the delkeys must be sorted in order to perform no redundant operations
    // that means, when we have reach the end of the array of delKeys,
    // we know that we don't have search the next leaf.
    public int keyedLeafRemove(String[] delKeys, Vector[] delVals) {

         boolean stop = false;
         int leafIndex = 0;
         int delIndex = 0; 
         while (!stop) {
              int res = _keys[leafIndex].compareTo(delKeys[delIndex]);

              // if there is a matching key, then do the deletion
              if (res ==0) {
                  for (int j = 0; j < delVals[delIndex].size(); j++) {
                      for (int k =0; k < _values[leafIndex].size(); k++) {
                         if (_values[leafIndex].elementAt(k) == delVals[j])
                            _values[leafIndex].removeElementAt(k);
                      }
                  }
                  delIndex++;
                  leafIndex++;
              }// end-if
              // if the current key is already greater than 
              else if (res > 0) {
                    delIndex++;
              } //end-else

              if (leafIndex >= _arity) 
                  stop = true;
              if (delIndex >= delKeys.length) 
                  stop = true;
          }// end-while
         
          if (delIndex >= delKeys.length)
              return -1;
          else
              return 1;
    }
                    
    /*
        for (i =0; i < totalSize/2;i++) {
            newLeaf._keys.addElements(_keys.elementAt(0));
            newLeaf._values.addElements(_values.elementAt(0));
            _keys.removeElementAt(0);
            _values.removeElementAt(0);
        }

        Leaf newLeaf = new Leaf(newKeys,newValues);
        return newLeaf;
    } 
    */

    // insert the new entry into the leaf, if the key already exist,
    // then add the values into the existing values
    // else create a new entry in the leaf, if the leaf is full, then
    // split the leaf by creating a new leaf node and put half of the
    // entries in the original leaf into the new leaf.  Copy the middle
    // key and the pointer to the new leaf as a new NodeEntry to be
    // inserted into the parent node.
    public NodeEntry insert(LeafEntry entry){ 

System.out.println("~~~~~~~~~~~~~ LEAF INSERT~~~~~~~~~~~");

      int position = getPosition(entry._key);

System.out.println("~~~~~~~~~AFTER GET POSITION ");
System.out.println("position found = "+position);
    
System.out.println("now insert..........");


      if (position < MAX) {

          if (_keys[position] == null) {
              orderedInsert(entry,position);
              _arity++;
              return null;
          }

          if (_keys[position].compareTo(entry._key) == 0)  {
System.out.println("add into exisitng,done!!! ");

              for (int i = 0; i < (entry._values).size();i++)

                  _values[position].addElement((entry._values).elementAt(i));
System.out.println("~~~~~~~~~~~~~END INSERT LEAF");
              return null;
          }    
          else {
              if (_arity < MAX) {

System.out.println(" ordered insert");
                  orderedInsert(entry,position);
                  _arity++;
                  return null;
              }
              else 
System.out.println(" NEED TO SPLIT~~~~~~~~~~");
                  return split(entry,position);
          }
      }



      return split(entry,position);
    }
             
    /* insert the promoted entry to the current node. */
    public void orderedInsert(LeafEntry entry, int position) {


        // move the entries have key greater than the key to be
        // inserted one position back.
        // the entry to be added is the key plus the pointer behind it
        // which is at the position of 1 more than that of the key.
        // since the number of pointers is 1 more than keys 
        // and we need to move only the set of key and pointer
        // after the key and pointer to be inserted thus the position
        // of the pointers to be moved is 1 more than that of the keys.


        // move the data starting from "position" 1 space back to
        // make room for the new entry to be inserted at "position"
        moveDataBackward(position);
       
         // insert the entry into the appropriate position
        insertIntoPosition(position,entry);
    }         

    // move the data starting from "postion" 1 pace back to make room
    public void moveDataBackward(int position) {
      
         for (int i = _arity; i > position; i--) {
            _keys[i] = _keys[i-1];
            _values[i] = _values[i-1];
        }
    }

    // insert an leaf entry into its corresponding position
    public void insertIntoPosition(int position,LeafEntry entry) {
        _keys[position] = entry._key;
        _values[position] = entry._values;

    }
          
    /* search for the ordered position where the new node entry should
     * be places */
    private int getPosition(String key) {


System.out.println("~~~~~~~~~~~~~~~~~ IN GET POSITION");
System.out.println("insert key = " + key);

        int pos = 0;

System.out.println("ARITY = " + _arity);

        for (int i = 0; i < _arity; i++) {

System.out.print("  key : "+ _keys[i]);
            if (_keys[i].compareTo(key) >= 0) 
                return i;
        }

System.out.println("");
System.out.println("END OF GET POSTION ~~~~~~");
        return _arity;
    }


}

        
      
          
    

    
