//fileInfoV: the vector of initial files
//ht[0]~ht[5]: the hashtables of different metadata

//methods:
//native long[] getInfo(String name)
//public void buildAllMeta()
//public void modifyFileMeta()
//private void insertInHashTable(Hashtable ht, String obj, int fileNum)
//private void addInSort(LinkedList list, String obj, int fileNum)
//private String mapIDtoName(int id)
//private void getIDNameMap()
//private Vector bulkLoadHelp1(Hashtable hashtab) //for strings
//private Vector bulkLoadHelp2(Hashtable hashtab) //for integers

//classes involved
//public class LeafEntry 
//public class FileInfoEntry
//class MapNode implements Comparable

import java.util.*;
import java.io.*;

//To get metadata of a file
//buf[0]: uid of file owner
//buf[1]: gid of file owner
//buf[2]: file size in bytes
//buf[3]: time of last access
//buf[4]: time of last data modification
//buf[5]: time of last file status change

//meta[0]~[1]: Strings of user names
//meta[2]~[6]: Strings of time, size

public class MetaData implements Serializable{

    private int fileNO;
    private String pathName;
    private Hashtable[] ht = new Hashtable[6];

    //convert idNameMap to an array in order for the convenience 
    //for searching a certain uid
    //private MapNode[] mapArray;


    //Native method declaration
    native long[] getInfo(String name);
    //Load the library
    static {
        System.loadLibrary("metadata");
    }

    public void buildAllMeta(){
	long[] buf = new long[6];
	String[] meta = new String[6];

	getIDNameMap();

	try{
	    String filename = "userIDNameMap";
	    FileOutputStream fos  = new FileOutputStream(filename);
	    ObjectOutputStream out = new ObjectOutputStream(fos);
	    out.writeObject(ourDB.idNameMap);
	    out.flush();
	    out.close();
	}catch(IOException e){
	    System.out.println(e);
	}
	/*
	try{
	    String filename = "mapArray";
	    FileOutputStream fos  = new FileOutputStream(filename);
	    ObjectOutputStream out = new ObjectOutputStream(fos);
	    out.writeObject(ourDB.mapArray);
	    out.flush();
	    out.close();
	}catch(IOException e){
	    System.out.println(e);
	}
	*/
	//initialize hashtables
	for (int i=0; i<6; i++)
	    ht[i] = new Hashtable();

	//scan through the fileInfoV to get the metadata of each file
	for (fileNO = 0; fileNO < ourDB.fileInfoV.size(); fileNO++){
	    if(ourDB.fileInfoV.get(fileNO)!=null){
		pathName = ((FileInfoEntry)(ourDB.fileInfoV.get(fileNO))).filePathName;
		buf = getInfo(pathName);

		//preparation for updating, store path, fileID and modification
		//time into ourDB.oldFiles
		ourDB.oldFiles.add(new MidModEntry(pathName, fileNO, buf[4]));
		try{
		    //String filename = "oldfileswithmodtime";
		    FileOutputStream fos  = new FileOutputStream("oldFiles");
		    ObjectOutputStream out = new ObjectOutputStream(fos);
		    out.writeObject(ourDB.oldFiles);
		    out.flush();
		    out.close();
		}catch(IOException e){
		    System.out.println(e);
		}

		//insert all metadata into hashtables
		for (int i=0; i<2; i++){
		    String uname = mapIDtoName((int)buf[i]);
		    meta[i] = new String(uname);
		    insertInHashTable(ht[i], meta[i], fileNO);
		}
		for (int i=2; i<buf.length; i++){
		    Integer[] metaTemp = new Integer[4]; 
		    metaTemp[i-2] = new Integer((int)buf[i]); 
		    meta[i] = metaTemp[i-2].toString();
		    insertInHashTable(ht[i], meta[i], fileNO);
		}
	    }
	}

	//convert the hashtables to vectors in order to do bulk loading
	//Vector[] metaForBL = new Vector[6];
	for (int i=0; i<2; i++){
	    System.out.println("Vector "+i+" for bulk loading...");
	    ourDB.metaForBL[i] = bulkLoadHelp1(ht[i]);
	    //print out the content of the vector
	    for (int j=0; j < ourDB.metaForBL[i].size(); j++){
		LeafEntry tmpEntry = (LeafEntry)(ourDB.metaForBL[i].get(j));
		System.out.print(tmpEntry._key);
		for(int m=0; m<((Vector)(tmpEntry._values)).size(); m++)
		    System.out.print(" "+((Integer)(tmpEntry._values.get(m))).intValue());
		System.out.println();
	    }
	}
	for (int i=2; i<6; i++){
	    System.out.println("Vector "+i+" for bulk loading...");
	    ourDB.metaForBL[i] = bulkLoadHelp2(ht[i]);
	    //print out the content of the vector
	    for (int j=0; j < ourDB.metaForBL[i].size(); j++){
		LeafEntry tmpEntry = (LeafEntry)(ourDB.metaForBL[i].get(j));
		System.out.print(tmpEntry._key);
		for(int m=0; m<((Vector)(tmpEntry._values)).size(); m++)
		    System.out.print(" "+((Integer)(tmpEntry._values.get(m))).intValue());
		System.out.println();
	    }
	}
    }

    //This part need to be modified, three modified vectors are provided
    //instead of only one 
    
    public void modifyFileMeta(){
	int fileNumber;
	long[] buf = new long[6];
	String[] meta = new String[6];
	Integer[] metaTemp = new Integer[4];
	ht = new Hashtable[6];

	for (int i = 0; i<ourDB.modifiedFiles.size(); i++){
	    if(ourDB.modifiedFiles.get(fileNO)!=null){
		pathName = ((ModFileEntry)(ourDB.modifiedFiles.get(i))).path;
		fileNumber = ((ModFileEntry)(ourDB.modifiedFiles.get(i))).fileNumber;
		buf = getInfo(pathName);
		for (i=0; i<2; i++){
		    String uname = mapIDtoName((int)buf[i]);
		    meta[i] = new String(uname);
		    //temporary solution, should pass a vector of filenums
		    Vector nums = new Vector();
		    nums.add(new Integer(fileNumber));
		    ourDB.bptreeMeta[i].insert(meta[i], nums);
		}
		for (i=2; i<buf.length; i++){
		    metaTemp = new Integer[4]; 
		    metaTemp[i-2] = new Integer((int)buf[i]); 
		    meta[i] = metaTemp[i-2].toString();
		    //temporary solution
		    Vector nums = new Vector();
		    nums.add(new Integer(fileNumber));
		    ourDB.bptreeMeta[i].insert(meta[i], nums);
		}
	    }
	}

	for (int i = 0; i<ourDB.createdFiles.size(); i++){
	    if(ourDB.createdFiles.get(fileNO)!=null){
		pathName = ((ModFileEntry)(ourDB.createdFiles.get(i))).path;
		fileNumber = ((ModFileEntry)(ourDB.createdFiles.get(i))).fileNumber;
		buf = getInfo(pathName);
		for (i=0; i<2; i++){
		    String uname = mapIDtoName((int)buf[i]);
		    meta[i] = new String(uname);
		    Vector nums = new Vector();
		    nums.add(new Integer(fileNumber));
		    ourDB.bptreeMeta[i].insert(meta[i], nums);
		}
		for (i=2; i<buf.length; i++){
		    metaTemp = new Integer[4]; 
		    metaTemp[i-2] = new Integer((int)buf[i]); 
		    meta[i] = metaTemp[i-2].toString();
		    Vector nums = new Vector();
		    nums.add(new Integer(fileNumber));
		    ourDB.bptreeMeta[i].insert(meta[i], nums);
		}
	    }
	}

    }
    

    //insert the user name to the hashtables
    private void insertInHashTable(Hashtable ht, String obj, int fileNum){
	LinkedList list = (LinkedList)ht.get(obj);
	if (list == null){
	    list = new LinkedList();
	    ListNode n = new ListNode(obj);
	    n.add(fileNum);
	    list.add(n);
	}
	else addInSort(list, obj, fileNum);
	
	ht.put(obj, list);
    }

    //add the names into the linkedlist in sorted order of corresponding hash bucket.
    private void addInSort(LinkedList list, String obj, int fileNum){
	int size = list.size();
	boolean flag = false;

	for (int i=0; i<size; i++){
	    String listvalue = (String)(((ListNode)list.get(i)).name);
	    //int listnum = ((ListNode)list.get(i)).fileNumber;
	    if(obj.compareTo(listvalue) < 0){
		ListNode n = new ListNode(obj);
		n.add(fileNum);
		list.add(i, n);
		return;
		//flag = true;
	    }
	    else if(obj.compareTo(listvalue)==0){
		ListNode n = (ListNode)list.get(i);
		n.add(fileNum);
		return;
		//flag = true;
	    }
	}
	//if (!flag){
	    ListNode n = new ListNode(obj);
	    n.add(fileNum);
	    list.addLast(n);
	    //}
	return;
    }
    
    private String mapIDtoName(int id){
	String name;
	int pos;
	Integer ID = new Integer(id);
	if(ourDB.mapArray==null){
	    System.out.println("mapArray = null");
	    return null;
	}
	else{ 
	    pos = Arrays.binarySearch(ourDB.mapArray, ID);
	    return(((MapNode)(ourDB.mapArray[pos])).name);
	}
    }

    private void getIDNameMap(){
	int uid;
	String uname;
	//idNameMap = new Vector();
	try{
	    BufferedReader in = new BufferedReader(new FileReader(new File("/etc/passwd")));
	    Object ref;
	    //delimitors to seperate words
	    String delims = ":";
	    int line = 0;
	    boolean flag = false;

	    do {
		ref = in.readLine();
		line ++;
		if (ref != null) {
		    StringTokenizer st = new StringTokenizer(new String((String)ref),delims);
		    
		    uname = st.nextToken();
		    String tmp = st.nextToken();
		    uid = Integer.parseInt(st.nextToken());
		    
		    MapNode mnode = new MapNode(uid, uname);
		    if(line == 1){
			ourDB.idNameMap.add(mnode);
		    }
		    else if(line < 10){
			flag = false;;
			for(int i=0; i<ourDB.idNameMap.size(); i++){
			    if (uid < ((MapNode)(ourDB.idNameMap.get(i))).id){
				ourDB.idNameMap.add(i, mnode);
				flag = true;
				break;
			    }
			}
			if(!flag){
			    ourDB.idNameMap.add(mnode);
			}
		    }
		    else{
			if(line < 20){
			}
			ourDB.idNameMap.add(mnode);
		    }
		}
	    }while(ref != null);
	    in.close();
	}catch(FileNotFoundException e) {
	    System.out.println("can't find file \\etc\\passwd ");
	}catch (IOException e) {
	    System.out.println("readLine() error");
	}

	Object[] tmpArray = ourDB.idNameMap.toArray();
	ourDB.mapArray = new MapNode[tmpArray.length];
	for(int i=0; i<tmpArray.length; i++)
	    ourDB.mapArray[i] = (MapNode)(tmpArray[i]);
    }

    private Vector bulkLoadHelp1(Hashtable hashtab){
	Vector vt = new Vector();
	Hashtable ht = hashtab;
	boolean flag = false;

	//get the keys of the hashtable
	Enumeration enum = ht.keys();
	while(enum.hasMoreElements()){
	    String element = (String)enum.nextElement();
	    //get the linkedlist which a certain key refers
	    LinkedList list = (LinkedList)(ht.get(element));
	    for (int i=0; i<list.size(); i++){
		ListNode listnode = (ListNode)list.get(i);
		String filename = (String)listnode.name;
		Vector filenums = listnode.fileNums;
		LeafEntry lfEntry = new LeafEntry(filename, filenums);
		if(vt.size()==0)
		    vt.add(lfEntry);
		else if(filename.compareTo((String)(((LeafEntry)(vt.get(vt.size()-1))))._key)>0)
		    vt.add(lfEntry);
		else{
		    for (int index = 0; index<vt.size(); index++){
			LeafEntry tmpEntry = (LeafEntry)(vt.get(index));
			if (filename.compareTo((String)(tmpEntry._key))<0){
			    vt.add(index, lfEntry);
			    break;
			}
		    }
		}
	    }
	}
	return vt;
    }


    private Vector bulkLoadHelp2(Hashtable hashtab){
	Vector vt = new Vector();
	Hashtable ht = hashtab;
	boolean flag = false;

	//get the keys of the hashtable
	Enumeration enum = ht.keys();
	while(enum.hasMoreElements()){
	    String element = (String)enum.nextElement();
	    //get the linkedlist which a certain key refers
	    LinkedList list = (LinkedList)(ht.get(element));
	    for (int i=0; i<list.size(); i++){
		ListNode listnode = (ListNode)list.get(i);
		//store the Integer as a string in leaf entry
		String filename = ((String)listnode.name);
		Vector filenums = listnode.fileNums;
		LeafEntry lfEntry = new LeafEntry(filename, filenums);
		if(vt.size()==0)
		    vt.add(lfEntry);
		else{
		    int one = Integer.parseInt(filename);
		    int two = Integer.parseInt((String)(((LeafEntry)(vt.lastElement()))._key));
		    if(one > two)
			vt.add(lfEntry);
		
		    else{
			for (int index = 0; index<vt.size(); index++){
			    LeafEntry tmpEntry = (LeafEntry)(vt.get(index));
			    int three = Integer.parseInt((String)(tmpEntry._key));
			    if (one < three){
				vt.add(index, lfEntry);
				break;
			    }
			}
		    }
		}
	    }
	}
	return vt;
    }
}




