import java.io.*;
import java.util.*;
// This class is used when updating the indexing every night.
// Three modified file vectors will be created, one for deleted files,
// one for modified files and one for created files.

// Vectors to export:
// deletedFiles
// modifiedFiles
// createdFiles


//methods:
//native long[] getModTime(String name)
//public ModifiedFiles()
//public void modifyFileV()
//private void getAllFiles()
//private void getAllFilesHelp(File subroot)

//private classes
//class ModFileEntry
//class MidModEntry

public class ModifiedFiles implements Serializable{

    //Native method declaration
    native long[] getModTime(String name);
    //Load the library
    static {
        System.loadLibrary("modtime");
    }

    private String root;

    public ModifiedFiles(String s){
	root = s;
    }

    //to be called every night after the first run
    public void modifyFileV(){
	String newFilePath;
	String oldFilePath;
	
	int i, j;
	
	try{
	    FileInputStream fis = new FileInputStream("writetemp");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.fileInfoV = (Vector)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}

	System.out.println("\n***** FileInfoV *****");
	for (i = 0; i < ourDB.fileInfoV.size(); i++){
	    FileInfoEntry entry = (FileInfoEntry)(ourDB.fileInfoV.get(i));
	    System.out.println(i +") "+entry.filePathName);
	    System.out.println(entry.fileType);
	}

	try{
	    FileInputStream fis = new FileInputStream("oldFiles");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.oldFiles = (Vector)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}

	try{
	    FileInputStream fis = new FileInputStream("userIDNameMap");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.idNameMap = (Vector)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	
	Object[] tmpArray = ourDB.idNameMap.toArray();
	ourDB.mapArray = new MapNode[tmpArray.length];
	for(i=0; i<tmpArray.length; i++)
	    ourDB.mapArray[i] = (MapNode)(tmpArray[i]);
	
	/*
	try{
	    FileInputStream fis = new FileInputStream("mapArray");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    mapArray = (MapNode[])(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	*/

	//get words index
	try{
	    FileInputStream fis = new FileInputStream("TEXT_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeText = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	//System.out.println("TEXT_INDEX after reload.");
	//ourDB.bptreeText.leafPrint();

	//get metadata indices
	//META_OWNER_INDEX
	try{
	    FileInputStream fis = new FileInputStream("META_OWNER_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeMeta[0] = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	//META_GROUP_INDEX
	try{
	    FileInputStream fis = new FileInputStream("META_GROUP_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeMeta[1] = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	//META_SIZE_INDEX
	try{
	    FileInputStream fis = new FileInputStream("META_SIZE_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeMeta[2] = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	//META_ACCESST_INDEX
	try{
	    FileInputStream fis = new FileInputStream("META_ACCESST_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeMeta[3] = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	//META_MODT_INDEX
	try{
	    FileInputStream fis = new FileInputStream("META_MODT_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeMeta[4] = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}
	//META_CHANGET_INDEX
	try{
	    FileInputStream fis = new FileInputStream("META_CHANGET_INDEX");
	    ObjectInputStream in = new ObjectInputStream(fis);
	    ourDB.bptreeMeta[5] = (BPlusTree)(in.readObject());
	    in.close();
	}catch(Exception e){
	    System.out.println(e);
	}

	//ourDB.newFiles = new Vector();
	//fill in the newFiles vector with all the files retrieved
	getAllFiles();

	for(i=0; i<ourDB.newFiles.size(); i++){
	    boolean find = false;
	    MidModEntry newEntry = (MidModEntry)(ourDB.newFiles.get(i));
	    //System.out.println("new entry: "+newEntry);
	    newFilePath = newEntry.path;

	    //System.out.println("new file path "+i+": "+newFilePath);
	    //j is file number
	    for(j=0; j<ourDB.oldFiles.size(); j++){
		MidModEntry oldEntry =(MidModEntry)(ourDB.oldFiles.get(j));
		oldFilePath = oldEntry.path;
		if(oldFilePath.equals(newFilePath)){
		    find = true;
		    //System.out.println("old path = new path");
		    long newFileTime = newEntry.modTime;
		    long oldFileTime = oldEntry.modTime;
		    //to see if the modification time has changed
		    if (oldFileTime!=newFileTime){
			//System.out.println("oldFileTime != newFileTime");
			ModFileEntry mfEntry = new ModFileEntry(newFilePath, j);
			ourDB.modifiedFiles.add(mfEntry);
		    }
		    newEntry.setFileNO(j);
		    //System.out.println("file number of new entry set to "+j);
		    oldEntry.mark = true;
		    //System.out.println("mark of old entry set to true");
		    break;
		}
	    }
	    //not found in old file list, so it's a new file
	    if(!find){
		ModFileEntry mfEntry = new ModFileEntry(newFilePath);
		ourDB.createdFiles.add(mfEntry);
	    }
	}   
	
	//scan through the old file list to find unmarked files
	for(j=0; j<ourDB.oldFiles.size(); j++){
	    MidModEntry oldEntry =(MidModEntry)(ourDB.oldFiles.get(j));
	    oldFilePath = oldEntry.path;
	    if(oldEntry.mark==false){
		ModFileEntry mfEntry = new ModFileEntry(oldFilePath, j);
		ourDB.deletedFiles.add(mfEntry);
	    }
	}

	
	//delete the files in fileInfoV
	for(i=0; i< ourDB.deletedFiles.size(); i++){
	    int fn = ((ModFileEntry)(ourDB.deletedFiles.get(i))).fileNumber;
	    boolean del = ourDB.delFileInfo(fn);
	    /*
	    FileInfoEntry entry = (FileInfoEntry)(ourDB.fileInfoV.get(fn));
	    System.out.println(fn + ") "+entry.filePathName + " " +entry.fileType);
	    */
	    if(!del)
		System.err.println("Cannot delete file "+((ModFileEntry)(ourDB.deletedFiles.get(i))).path);
	}

	//insert newly created files into fileInfoV and get filenumbers for them
	for(i=0; i<ourDB.createdFiles.size(); i++){
	    ModFileEntry entry = (ModFileEntry)(ourDB.createdFiles.get(i));
	    int fn = ourDB.insertFileInfo(entry.path);
	    //System.out.println(fn);
	    entry.setFileNO(fn);
	}
	
	System.out.println("\n***** FileInfoV(after mod) *****");
	for (i = 0; i < ourDB.fileInfoV.size(); i++){
	    FileInfoEntry entry = (FileInfoEntry)(ourDB.fileInfoV.get(i));
	    System.out.println(i +") "+entry.filePathName);
	    System.out.println(entry.fileType);
	}


	//today's newFiles becomes tomorrow's oldFiles, is this done in ourDB???
	try{
	    String filename = "oldFiles";
	    FileOutputStream fos  = new FileOutputStream(filename);
	    ObjectOutputStream out = new ObjectOutputStream(fos);
	    out.writeObject(ourDB.oldFiles);
	    out.flush();
	    out.close();
	}catch(IOException e){
	    System.out.println(e);
	}

	//OK, now the three vectors all contain nodes with path and file number
    }    
	    
    

    //scan the file system and get all the files into a vector,
    //vector node contains filepath and modification time, 
    //filenumber to be set
    private void getAllFiles(){
	getAllFilesHelp(new File(root));
    }

    private void getAllFilesHelp(File subroot){
        /* the root is a file */
        if (subroot.isFile()) {
            // get the path 
            String path = subroot.getAbsolutePath();
	    long[] time = getModTime(path);
	    
            // add file info entry to the file info vector
            ourDB.newFiles.add(new MidModEntry(path, time[0])); 
	}
        else{
            String[] file_list = subroot.list();
                
            for (int i=0; i<file_list.length; i++) 
                getAllFilesHelp(new File(subroot,file_list[i]));
        }
    }
}



