/* this file will cache current query result and some history result for 
 * further query and going back.
 * resultTree is a vector of vector of QueryResultEntry
 */

import java.util.*;

public class QueryResultTree {

    public static Vector resultTree = new Vector();

    public static int currQueryResult = -1;

    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    // find the corresponding file id or dir type
    // if no matching, return Integer.MAX_VALUE 
    public static int search(String name) {

	// if the name starts with '/' or "./", it refers to real dir
	if (name.startsWith("/") || name.startsWith("./"))
	    return Integer.MAX_VALUE;

	// the vector of current query result
       	SingleQueryResult curr = (SingleQueryResult)(resultTree.elementAt
						     (currQueryResult));
	return curr.search(name);
    }


    // search whether a dir has a child
    public static int searchChild(String name) {
	
	// the vector of current query result
       	SingleQueryResult curr = (SingleQueryResult)(resultTree.elementAt
						     (currQueryResult));
	return curr.searchChild(name);
    }


    // search the path of directory
    public static int searchPath(String name) {
	
	

	// the vector of current query result
       	SingleQueryResult curr = (SingleQueryResult)(resultTree.elementAt
						     (currQueryResult));
	return curr.searchPath(name);
    }


    // list the entries of current query result
    public static void listCurr() {

	((SingleQueryResult)(resultTree.elementAt(currQueryResult))).list();
    }


    // temporily build a query result tree to debug
    public static void build() {

	SingleQueryResult q1 = new SingleQueryResult();
	(q1.singleResult).addElement(new QueryResultEntry("a",4,-1));
	(q1.singleResult).addElement(new QueryResultEntry("d1",-1,1));
	(q1.singleResult).addElement(new QueryResultEntry("d2",-2,2));
	SingleQueryResult q2 = new SingleQueryResult();
	(q2.singleResult).addElement(new QueryResultEntry("c",4,-1));
	(q2.singleResult).addElement(new QueryResultEntry("e",5,-1));
	(q2.singleResult).addElement(new QueryResultEntry("f",6,-1));
	q2.parent = 0;
	SingleQueryResult q3 = new SingleQueryResult();
	(q3.singleResult).addElement(new QueryResultEntry("g",7,-1));
	(q3.singleResult).addElement(new QueryResultEntry("d3",-1,-1));
	(q3.singleResult).addElement(new QueryResultEntry("h",8,-1));
	q3.parent = 0;

	resultTree.addElement(q1);
	resultTree.addElement(q2);
	resultTree.addElement(q3);

	dump(resultTree);
    }

	
    // dump the contents of the result tree
    private static void dump(Vector vec) {

	for (int i=0; i<resultTree.size(); i++) {
	    int len = ((SingleQueryResult)(resultTree.elementAt(i))).singleResult.size();
	    System.out.print("Parent: ");
	    System.out.println(((SingleQueryResult)(resultTree.elementAt(i))).parent);

	    Vector v = ((SingleQueryResult)(resultTree.elementAt(i))).singleResult;
	    for (int j=0; j<len; j++) {
		System.out.print("Name: "+((QueryResultEntry)v.elementAt(j)).name+ " ");
		System.out.print("ID: "+((QueryResultEntry)v.elementAt(j)).fileNumOrDirType + " ");
		System.out.print("Child: "+((QueryResultEntry)v.elementAt(j)).child + " ");
	    }

	    System.out.println();
	}

	System.out.println();
	System.out.println("Current Query Result: " + currQueryResult);
    }
	

    // print some element of the resultTree which is pointed by pt
    public static void printResult(int pt) {

	if (0 > pt) {
	    System.out.println("ERROR: there is no query result in cache.");
	    return;
	}
	int printlen = 0;
	int len = ((SingleQueryResult)(resultTree.elementAt(pt))).singleResult.size();
	System.out.println("result length = " + len);
	Vector v = ((SingleQueryResult)(resultTree.elementAt(pt))).singleResult;
	for (int j=0; j<len; j++) {
	    System.out.print(((QueryResultEntry)v.elementAt(j)).name);
	    printlen += (((QueryResultEntry)v.elementAt(j)).name).length() + 6;
	    if (0 >((QueryResultEntry)v.elementAt(j)).fileNumOrDirType)
		System.out.print("/      ");
	    else 
		System.out.print("      ");
	    if (65 < printlen) {
		printlen = 0;
		System.out.println();
	    }

	}

	System.out.println();
    }

    // renew a new tree for a new query
    public static void renew() {
	
	// remove all the element from the resultTree vector and set 
	// currQueryResult as -1
	resultTree.removeAllElements();
	currQueryResult = -1;
    }


}
