//export a vector for bulk loading

//fileInfoV: the vector of initial files
//modifiedFileV: the vector of modified files
//fullTextH: the hashtable of plain words

//methods:
//public WordExtractor()
//public void buildAllWord()
//public void modifyFileWord()
//private void scanFile(String path, int fileNum)
//private boolean isUseful(String w)
//private boolean isAccessed(String w, int fileNum)
//private void insertInHashTable(String w, int fileNum)
//private void addInSort(LinkedList list, String w, int fileNum)
//private Vector bulkLoadHelp(Hashtable hashtab)

//class involved
//class LeafEntry

import java.io.*;
import java.util.*;

public class WordExtractor implements Serializable{

    //the unique number to identify file;
    private int fileNO;
    //correspondent file path to the fileNO according to the fileInfoV; 
    private String pathName;
    //file type
    private String type;
    //the word taken from the text, which is transformed to lower case;
    private String word;
    //the hashtable to store the words extracted from the file
    private Hashtable fullTextH;

    public WordExtractor(){
    }

    //To extract the words from all the files and map them into the hashtable
    public void buildAllWord(){
	String file;
	//Since buildAllWord and modifyWord need to use different hashtables, can I 
	//initialize fullTextH here and then in modifyFileWord again?
	fullTextH = new Hashtable();

	//is the index of the vector the fileID??
	for (fileNO = 0; fileNO < ourDB.fileInfoV.size(); fileNO++){
	    FileInfoEntry entry = (FileInfoEntry)(ourDB.fileInfoV.get(fileNO));
	    pathName = entry.filePathName;
	    type = entry.fileType;

	    if((entry != null)&&((type.equals("email"))||(type.equals("java"))||(type.equals("c"))||(type.equals("c++"))||(type.equals("txt"))||(type.equals("h")))){
		//System.out.println("file type is txt");
		scanFile(pathName, fileNO);
	    }
	    else if((entry != null) && (type.equals("pdf"))){
		//System.out.println("file type is pdf");
		file = convertpdf(pathName);
		System.out.println("convertpdf returns "+file);
		scanFile(file, fileNO);
	    }
	    else if((entry != null) && (type.equals("ps"))){
		//System.out.println("file type is ps");
		file = convertps(pathName);
		scanFile(file, fileNO);
	    }
	    else
		continue;
	}
	
	// This vector is used only for bulk loading. The node of the vector is actually the node of the
	//linkedlists of the hashtable
	ourDB.wordsForBL = bulkLoadHelp(fullTextH);
	
    }


    //I have a modifiedFiles vector and a createdFiles vector which hold all the 
    //modified files of the day so this method is to update the Btree for all the modified 
    //files, not just one build hashtables for modified files and insert the words 
    //into the B+tree.
    
    public void modifyFileWord(){
	int fileNumber;
	String file;

	fullTextH = new Hashtable();
	for (int i = 0; i < ourDB.modifiedFiles.size(); i++){
	    ModFileEntry entry = (ModFileEntry)(ourDB.modifiedFiles.get(i));
	    pathName = entry.path;
	    fileNumber = entry.fileNumber;
	    type = ((FileInfoEntry)(ourDB.fileInfoV.get(fileNumber))).fileType;
	
	    if((entry != null)&&((type.equals("email"))||(type.equals("java"))||(type.equals("c"))||(type.equals("c++"))||(type.equals("txt"))||(type.equals("h")))){
		//System.out.println("file type is txt");
		scanFile(pathName, fileNumber);
	    }
	    else if((entry != null) && (type.equals("pdf"))){
		//System.out.println("file type is pdf");
		file = convertpdf(pathName);
		System.out.println("convertpdf returns "+file);
		scanFile(file, fileNumber);
	    }
	    else if((entry != null) && (type.equals("ps"))){
		//System.out.println("file type is ps");
		file = convertps(pathName);
		scanFile(file, fileNumber);
	    }
	    else
		continue;
	}

	for (int i = 0; i < ourDB.createdFiles.size(); i++){
	    ModFileEntry entry = (ModFileEntry)(ourDB.createdFiles.get(i));
	    pathName = entry.path;
	    fileNumber = entry.fileNumber;
	    type = ((FileInfoEntry)(ourDB.fileInfoV.get(fileNumber))).fileType;
	
	    if((entry != null)&&((type.equals("email"))||(type.equals("java"))||(type.equals("c"))||(type.equals("c++"))||(type.equals("txt"))||(type.equals("h")))){
		//System.out.println("file type is txt");
		scanFile(pathName, fileNumber);
	    }
	    else if((entry != null) && (type.equals("pdf"))){
		//System.out.println("file type is pdf");
		file = convertpdf(pathName);
		System.out.println("convertpdf returns "+file);
		scanFile(file, fileNumber);
	    }
	    else if((entry != null) && (type.equals("ps"))){
		//System.out.println("file type is ps");
		file = convertps(pathName);
		scanFile(file, fileNumber);
	    }
	    else
		continue;
	}

	Vector vecForMod = bulkLoadHelp(fullTextH);
	for (int j = 0; j < vecForMod.size(); j++){
	    LeafEntry le = (LeafEntry)(vecForMod.get(j));
	    System.out.println("In WordExtractor: modifyFileWord, insert");
	    System.out.print(le._key+" ");
	    for(int i=0; i<le._values.size(); i++)
		System.out.print(le._values.get(i)+" " );
	    System.out.println();
	    ourDB.bptreeText.insert(le._key, le._values);
	}
    }
	/*
	    for(Enumeration e = fullTextH.keys(); e.hasMoreElements();){
	    String key = (String)e.nextElement();
	    LinkedList list = (LinkedList)fullTextH.get(key);
	    for(int i = 0; i < list.size(); i++){
		ListNode node = (ListNode)list.get(i);
		String name = (String)(node.name);
		Vector nums = node.fileNums;
		
		bptree.insert(name, nums);
	    }
	}
    }




	
	fullTextH = new Hashtable();
	for (int i = 0; i < ourDB.createdFiles.size(); i++){
	    if(ourDB.createdFiles.get(i) != null){
		pathName = ((ModFileEntry)(ourDB.createdFiles.get(i))).path;
		fileNumber = ((ModFileEntry)(ourDB.createdFiles.get(i))).fileNumber;
		scanFile(pathName, fileNumber);
	
		for(Enumeration e = fullTextH.keys(); e.hasMoreElements();){
		    String key = (String)e.nextElement();
		    LinkedList list = (LinkedList)fullTextH.get(key);
		    for(int i = 0; i < list.size(); i++){
			ListNode node = (ListNode)list.get(i);
			String name = (String)(node.name);
			Vector nums = node.fileNums;

			bptree.insert(name, nums);
		    }
		}
	    }
	}
    }
    
	*/
    //scan the file to retrieve useful words and insert them to the hashtable 
    private void scanFile(String path, int fileNum){

	System.out.println("scan file "+path);

	boolean isLetter = false;
	boolean isUseful = false;

	try{
	    //???should pass in the whole path of the file? 
	    BufferedReader in = new BufferedReader(new FileReader(new File(path)));
	    Object ref;
	    //delimitors to seperate words
	    String delims = " ,./<>?;:[]{}()_+|*&^%$#@!~`\\=-\" \t";
	    
	    ///System.out.println("path is "+path);
	    do {
		ref = in.readLine();
		if (ref != null) {
		    StringTokenizer st = new StringTokenizer(new String((String)ref),delims);
		    while (st.hasMoreTokens()){
			String word = st.nextToken().toLowerCase();
			//System.out.println("WORD "+word +" in FILE "+path);
			char firstLetter = word.charAt(0);
			isLetter = false;
			//get rid of numerical words
			if((firstLetter >= 'a')&&(firstLetter <= 'z'))
			    isLetter = true;
			isUseful = isUseful(word);
			//get rid of useless words
			if (isLetter && isUseful){
			    //get rid of the postfix "'s"
			    if (word.endsWith("'s")){
				int endIndex = word.indexOf("'");
				word = word.substring(0, endIndex);
			    }
			    boolean isaccessed = isAccessed(word, fileNum);
			    //get rid of those words that have been accessed
			    if (!isaccessed){
				//insert this word to the hash table
				insertInHashTable(word, fileNum);
				
			    }
			}
		    }
		}
	    }while(ref != null);
	    in.close();
	}catch(FileNotFoundException e) {
	    System.out.println("can't find file \""+path+"\"");
	}catch (IOException e) {
	    System.out.println("readLine() error");
	}
    }

    //to get rid of the useless words
    private boolean isUseful(String w){
	try{
	BufferedReader infile = new BufferedReader(new FileReader("uselesswords.dat"));
	Object line;
	do {
	    line = infile.readLine();
	    if(line != null){
		StringTokenizer s = new StringTokenizer(new String((String)line));
		while(s.hasMoreTokens()){
		    String useless = s.nextToken();
		    //compare the word with the words in a existing file uselesswords.dat,
		    //if equal, then the word is ignored
		    if (w.equalsIgnoreCase(useless))
			return false;
		}
	    }
	}while (line != null);
	infile.close();
	}catch(FileNotFoundException e) {
	    System.out.println("can't find file!");
	} catch (IOException e) {
	    System.out.println("readLine() error");
	}
	return true;
    }

    //to judge if the word if already accessed
    private boolean isAccessed(String w, int fileNum){
	LinkedList list = (LinkedList) fullTextH.get(w);
	if(list == null)
	    return false;//not accessed;
	else{
	    for (int i = 0; i< list.size(); i++){
		ListNode node = (ListNode)list.get(i);
		if(w.equals(node.name) && node.fileNums.contains(new Integer(fileNum)))
		    return true;
	    }
	}
	return false;
    }

    //insert the word to the global hashtable
    private void insertInHashTable(String w, int fileNum){
	//ListNode node = new ListNode(w,fileNum);
	LinkedList list = (LinkedList)fullTextH.get(w);
	if (list == null){
	    ///System.out.println("The list is empty");
	    list = new LinkedList();
	    ListNode n = new ListNode(w);
	    n.add(fileNum);
	    list.add(n);
	}
	else addInSort(list, w, fileNum);
	
	fullTextH.put(w, list);

	list = (LinkedList)fullTextH.get(w);
    }

    //add the word to the linkedlist in sorted word order of corresponding hash bucket
    private void addInSort(LinkedList list, String w, int fileNum){
	int size = list.size();
	boolean flag = false;

	for (int i=0; i<size; i++){
	    String listword = (String)(((ListNode)list.get(i)).name);
	    //System.out.println("Comparing "+listword+" and "+w);

	    if(w.compareTo(listword) < 0){
		//System.out.println(w +" < "+listword);
		ListNode n = new ListNode(w);
		n.add(fileNum);
		list.add(i, n);
		return;
	    }
	    else if(w.compareTo(listword)==0){
		//System.out.println(w +" = " +listword);
		ListNode n = (ListNode)list.get(i);
		n.add(fileNum);
		return;
	    }
	}
	//System.out.println(w+" is the largest word of this list.");
	ListNode n = new ListNode(w);
	n.add(fileNum);
	list.addLast(n);
	return;
    }

    private Vector bulkLoadHelp(Hashtable hashtab){
	Vector vt = new Vector();
	Hashtable ht = hashtab;
	boolean flag = false;

	//get the keys of the hashtable
	Enumeration enum = ht.keys();
	while(enum.hasMoreElements()){
	    String element = (String)enum.nextElement();
	    //get the linkedlist which a certain key refers
	    LinkedList list = (LinkedList)(ht.get(element));
	    for (int i=0; i<list.size(); i++){
		ListNode listnode = (ListNode)list.get(i);
		String filename = (String)listnode.name;
		Vector filenums = listnode.fileNums;
		LeafEntry lfEntry = new LeafEntry(filename, filenums);
		if(vt.size()==0)
		    vt.add(lfEntry);
		else if(filename.compareTo((String)(((LeafEntry)(vt.get(vt.size()-1))))._key)>0)
		    vt.add(lfEntry);
		else{
		    for (int index = 0; index<vt.size(); index++){
			LeafEntry tmpEntry = (LeafEntry)(vt.get(index));
			if (filename.compareTo((String)(tmpEntry._key))<0){
			    vt.add(index, lfEntry);
			    break;
			}
		    }
		}
	    }
	}

	/*
	System.out.println("Print out all the words of the file in sorted order for bulk loading\n");
	for(int x = 0; x < vt.size(); x++){
	    LeafEntry entry = (LeafEntry)(vt.get(x));
	    
	    System.out.print((String)(entry._key));
	    for (int y = 0; y < entry._values.size(); y++)
		System.out.print(" "+((Integer)(entry._values.get(y))).intValue());
	    System.out.println("\n");
	}
	*/
	return vt;
    }

    private String convertps(String path){
	String newPath = path + ".txt";
	String c = "ps2ascii " + path +  " " + newPath;
	Thread t = new Thread(new Command(c));
	t.start();
	try{
	    t.join();
	}catch(InterruptedException e){
	    System.out.println("Interrupted Exception. Terminating...");
	    System.exit(0);
	}
	return newPath;
    }	

    private String convertpdf(String path){
	System.out.println("Enter converpdf");
	
	String newPath;

	if(path.endsWith(".pdf"))
	    newPath = path.substring(0,path.lastIndexOf('.'))+".txt";
	else
	    newPath = path + ".txt";
	String c = "pdftotext "+ path;
	Thread t = new Thread(new Command(c));
	t.start();
	try{
	    t.join();
	}catch(InterruptedException e){
	    System.out.println("Interrupted Exception. Terminating...");
	    System.exit(0);
	}
	return newPath;
    }

}

class Command implements Runnable{
    private String thisCommand;
    private Runtime r;
    private Process p;

    public Command(String s){
	thisCommand = s;
    }

    public void run(){
	r = Runtime.getRuntime();
	try{
	    p = r.exec(thisCommand);
	}catch(IOException e ){
	    System.out.println("Output problem. Terminating.");
	    System.exit(0);
	}
    }
}




    




